"""
Models tests of Django Flickrsets application.
"""
import unittest

from django import test
from django.core.urlresolvers import reverse

from flickrsets import constants
from flickrsets.models import Person
from flickrsets.models import Photo
from flickrsets.models import Photoset
from flickrsets.models import Tag
from flickrsets.tests.client import FakeClient


class PersonTest(test.TestCase):
    """
    Tests ``Person`` model.
    """
    fixtures = ['flickrsets/tests']

    def setUp(self):
        """
        Sets up the test.
        """
        self.person = Person.objects.get(
            flickr_id=constants.TEST_PERSON_FLICKR_ID)

    def test_unicode(self):
        """
        Tests ``__unicode__`` value.
        """
        self.assertEquals(self.person.__unicode__(), self.person.name)

    def test_get_absolute_url(self):
        """
        Tests ``get_absolute_url`` method.
        """
        url = reverse('flickrsets-person', args=[self.person.flickr_id])
        self.assertEquals(self.person.get_absolute_url(), url)

    def test_name(self):
        """
        Tests ``name`` property.
        """
        self.assertEquals(self.person.name, u'Gilles Fabio')
        
        person = Person.objects.create(
            flickr_id=u'FOOBAR',
            photos_url=self.person.photos_url,
            profile_url=self.person.profile_url,
            mobile_url=self.person.mobile_url)   
        self.assertEquals(person.name, u'FOOBAR')
        person.username = u'superfoobar'
        person.save()
        self.assertEquals(person.name, u'superfoobar')
        
        person.realname = u'Super Foo Bar'
        person.save()
        self.assertEquals(person.name, u'Super Foo Bar')
        
        person.delete()

    def test_default_gender(self):
        """
        Tests default ``gender`` field value.
        """
        self.assertEquals(self.person.gender, constants.FLICKR_DEFAULT_GENDER)
        
        person = Person.objects.create(
            flickr_id=u'FOOBAR',
            photos_url=self.person.photos_url,
            profile_url=self.person.profile_url,
            mobile_url=self.person.mobile_url)
        self.assertEquals(person.gender, constants.FLICKR_DEFAULT_GENDER)
        
        person.gender = constants.FLICKR_GENDER_OTHER
        person.save()
        self.assertEquals(person.gender, constants.FLICKR_GENDER_OTHER)
        
        person.delete()

    def test_buddy_icon_url(self):
        """
        Tests ``buddy_icon_url`` property.
        """
        url = u'http://farm%s.static.flickr.com/%s/buddyicons/%s.jpg' % (
            self.person.icon_farm,
            self.person.icon_server,
            self.person.flickr_id)
        self.assertEquals(self.person.buddy_icon_url, url)
        
        person = Person.objects.create(
            flickr_id=u'FOOBAR',
            photos_url=self.person.photos_url,
            profile_url=self.person.profile_url,
            mobile_url=self.person.mobile_url)
        default_url = u'http://www.flickr.com/images/buddyicon.jpg'            
        self.assertEquals(person.buddy_icon_url, default_url)
        
        person.delete()


class PersonManagerTest(unittest.TestCase):
    """
    Tests ``PersonManager`` manager.
    """
    FLICKR_ID = constants.TEST_PERSON_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_create_object_from_api(self):
        """
        Tests ``create_object_from_api`` method.
        """
        person = Person.objects.create_object_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        self.assertEquals(person.flickr_id, self.FLICKR_ID)
        self.assertEquals(person.name, u'Gilles Fabio')
        
        Person.objects.all().delete()
        self.assertEquals(Person.objects.all().count(), 0)

    def test_get_or_create_from_api(self):
        """
        Tests ``get_or_create_from_api`` method.
        """
        person, created = Person.objects.get_or_create_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        self.assertEquals(created, True)
        self.assertEquals(person.flickr_id, self.FLICKR_ID)
        self.assertEquals(person.name, u'Gilles Fabio')

        person, created = Person.objects.get_or_create_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        self.assertEquals(created, False)
        self.assertEquals(person.flickr_id, self.FLICKR_ID)
        self.assertEquals(person.name, u'Gilles Fabio')

        Person.objects.all().delete()
        self.assertEquals(Person.objects.all().count(), 0)


class PhotoTest(test.TestCase):
    """
    Tests ``Photo`` model.
    """
    fixtures = ['flickrsets/tests']

    def setUp(self):
        """
        Sets up the test.
        """
        self.photo = Photo.objects.get(
            flickr_id=constants.TEST_PHOTO_FLICKR_ID)

    def test_unicode(self):
        """
        Tests ``__unicode__`` method.
        """
        self.assertEquals(self.photo.__unicode__(), self.photo.title)

    def test_get_absolute_url(self):
        """
        Tests ``get_absolute_url`` method
        """
        url = reverse('flickrsets-photo', args=[self.photo.flickr_id])
        self.assertEquals(self.photo.get_absolute_url(), url)

    def test_image_source(self):
        """
        Tests ``image_source`` property and ``image_VERSION_source`` dynamic
        properties.
        """
        for abbrev, rname, name in constants.FLICKR_PHOTO_SOURCES:
            if abbrev != constants.FLICKR_PHOTO_SOURCE_MEDIUM:
                secret = self.photo.secret
                if abbrev == constants.FLICKR_PHOTO_SOURCE_ORIGINAL:
                    secret = self.photo.original_secret
                url = u'http://farm%s.static.flickr.com/%s/%s_%s_%s.jpg' % (
                    self.photo.farm,
                    self.photo.server,
                    self.photo.flickr_id,
                    secret,
                    abbrev)
                photo_url = getattr(self.photo, 'image_%s_source' % rname)
                self.assertEquals(photo_url, url)
                
        default_url = u'http://farm%s.static.flickr.com/%s/%s_%s.jpg' % (
            self.photo.farm,
            self.photo.server,
            self.photo.flickr_id,
            self.photo.secret)
        self.assertEquals(self.photo.image_source, default_url)

    def test_image_url(self):
        """
        Tests ``image_url`` property and ``image_VERSION_url`` dynamic
        properties.
        """
        for abbrev, rname, name in constants.FLICKR_PHOTO_URLS:
            url = u'%s%s/sizes/%s/' % (
                self.photo.owner.photos_url,
                self.photo.flickr_id,
                abbrev)
            photo_url = getattr(self.photo, 'image_%s_url' % rname)
            self.assertEquals(photo_url, url)
            
        default_url = u'%s%s/' % (
            self.photo.owner.photos_url,
            self.photo.flickr_id)
        self.assertEquals(self.photo.get_image_url(size='foo'), default_url)


class PhotoManagerTest(unittest.TestCase):
    """
    Tests ``PhotoManager`` manager.
    """
    FLICKR_ID = constants.TEST_PHOTO_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_create_object_from_api(self):
        """
        Tests ``create_object_from_api`` method.
        """
        photo = Photo.objects.create_object_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client,
            create_tags=False,
            save_exif_tags=False)
        self.assertEquals(photo.flickr_id, self.FLICKR_ID)
        self.assertEquals(photo.title, u'Mozart Opera Rock')

        Photo.objects.all().delete()
        self.assertEquals(Photo.objects.all().count(), 0)

    def test_get_or_create_from_api(self):
        """
        Tests ``get_or_create_from_api`` method.
        """
        photo, created = Photo.objects.get_or_create_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client,
            create_tags=False,
            save_exif_tags=False)
        self.assertEquals(created, True)
        self.assertEquals(photo.flickr_id, self.FLICKR_ID)
        self.assertEquals(photo.title, u'Mozart Opera Rock')

        photo, created = Photo.objects.get_or_create_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client,
            create_tags=False,
            save_exif_tags=False)
        self.assertEquals(created, False)
        self.assertEquals(photo.flickr_id, self.FLICKR_ID)
        self.assertEquals(photo.title, u'Mozart Opera Rock')

        Photo.objects.all().delete()
        self.assertEquals(Photo.objects.all().count(), 0)


class PhotosetTest(test.TestCase):
    """
    Tests ``Photoset`` model.
    """
    fixtures = ['flickrsets/tests']

    def setUp(self):
        """
        Sets up the test.
        """
        self.photoset = Photoset.objects.get(
            flickr_id=constants.TEST_PHOTOSET_FLICKR_ID)

    def test_unicode(self):
        """
        Tests ``__unicode__`` method.
        """
        self.assertEquals(self.photoset.__unicode__(), self.photoset.title)

    def test_get_absolute_url(self):
        """
        Tests ``get_absolute_url`` method.
        """
        url = reverse('flickrsets-photoset', args=[self.photoset.flickr_id])
        self.assertEquals(self.photoset.get_absolute_url(), url)

    def test_get_flickr_url(self):
        """
        Tests ``get_flickr_url`` method.
        """
        url = u'http://flickr.com/photos/%s/sets/%s/' % (
            self.photoset.owner.photos_url,
            self.photoset.flickr_id)
        self.assertEquals(self.photoset.get_flickr_url(), url)


class PhotosetManagerTest(unittest.TestCase):
    """
    Tests ``PhotosetManager`` manager.
    """
    FLICKR_ID = constants.TEST_PHOTOSET_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_create_object_from_api(self):
        """
        Tests ``create_object_from_api`` method.
        """
        photoset = Photoset.objects.create_object_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        self.assertEquals(photoset.flickr_id, self.FLICKR_ID)
        self.assertEquals(photoset.title, u'Misc')

        Photoset.objects.all().delete()
        self.assertEquals(Photoset.objects.all().count(), 0)

    def test_get_or_create_from_api(self):
        """
        Tests ``get_or_create_from_api`` method.
        """
        photoset, created = Photoset.objects.get_or_create_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        self.assertEquals(created, True)
        self.assertEquals(photoset.flickr_id, self.FLICKR_ID)
        self.assertEquals(photoset.title, u'Misc')

        photoset, created = Photoset.objects.get_or_create_from_api(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        self.assertEquals(created, False)
        self.assertEquals(photoset.flickr_id, self.FLICKR_ID)
        self.assertEquals(photoset.title, u'Misc')

        Photoset.objects.all().delete()
        self.assertEquals(Photoset.objects.all().count(), 0)


class TagTest(test.TestCase):
    """
    Tests ``Photo`` model.
    """
    fixtures = ['flickrsets/tests']

    def setUp(self):
        """
        Sets up the test.
        """
        self.tag = Tag.objects.get(raw=constants.TEST_TAG_RAW)

    def test_unicode(self):
        """
        Tests ``__unicode__``  method.
        """
        self.assertEquals(self.tag.__unicode__(), self.tag.name)

    def test_get_absolute_url(self):
        """
        Tests ``get_absolute_url`` method.
        """
        url = reverse('flickrsets-tag', args=[self.tag.raw])
        self.assertEquals(self.tag.get_absolute_url(), url)


class TagManagerTest(unittest.TestCase):
    """
    Tests ``TagManager`` manager.
    """
    PHOTO_FLICKR_ID = constants.TEST_PHOTO_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_create_objects_from_api(self):
        """
        Tests ``create_objects_from_api`` method.
        """
        photo, created = Photo.objects.get_or_create_from_api(
            flickr_id=self.PHOTO_FLICKR_ID,
            client=self.api_client,
            create_tags=False,
            save_exif_tags=False)
        self.assertEquals(created, True)
        self.assertEquals(photo.title, u'Mozart Opera Rock')
    
        Tag.objects.all().delete()
        self.assertEquals(Tag.objects.all().count(), 0)
           
        tags = Tag.objects.create_objects_from_api(
            photo=photo,
            client=self.api_client)   
        self.assertEquals(len(tags), 3)
        
        Tag.objects.all().delete()
        self.assertEquals(Tag.objects.all().count(), 0)
        
        Photo.objects.all().delete()
        self.assertEquals(Photo.objects.all().count(), 0)

    def test_filter_or_create_from_api(self):
        """
        Tests ``filter_or_create_from_api`` method.
        """
        photo, created = Photo.objects.get_or_create_from_api(
            flickr_id=self.PHOTO_FLICKR_ID,
            client=self.api_client,
            create_tags=False,
            save_exif_tags=False)
        self.assertEquals(created, True)
        self.assertEquals(photo.title, u'Mozart Opera Rock')
        
        Tag.objects.all().delete()
        self.assertEquals(Tag.objects.all().count(), 0)
        
        tags, created = Tag.objects.filter_or_create_from_api(
            photo=photo, 
            client=self.api_client)
        self.assertEquals(created, True)
        self.assertEquals(len(tags), 3)
        
        tags, created = Tag.objects.filter_or_create_from_api(
            photo=photo,
            client=self.api_client)   
        self.assertEquals(created, False)
        self.assertEquals(len(tags), 3)
        
        Tag.objects.all().delete()
        self.assertEquals(Tag.objects.all().count(), 0)

        Photo.objects.all().delete()
        self.assertEquals(Photo.objects.all().count(), 0)



"""
Manager classes of Django Flickrsets application.
"""
import logging

from django.db import models

log = logging.getLogger('flickrsets.managers')


class PersonManager(models.Manager):
    """
    Manager of ``Person`` model.
    """
    
    def get_or_create_from_api(self, flickr_id, client):
        """
        Returns a ``Person`` object or creates it from Flickr API.

        Takes two required arguments:
        
            * ``flickr_id``: Photo's Flickr ID
            * ``client``: ``FlickrClient`` instance
            
        Returns a tuple of ``(object, created)``, where ``created`` is a
        boolean specifying wether an object was created.
        """
        try:
            return self.get(flickr_id=flickr_id), False
        except self.model.DoesNotExist:
            obj = self.create_object_from_api(
                flickr_id=flickr_id, 
                client=client)
            return obj, True

    def create_object_from_api(self, flickr_id, client):
        """
        Creates a new ``Person`` object from Flickr API.
        
        Takes two required arguments:
        
            * ``flickr_id``: Photo's Flickr ID
            * ``client``: ``FlickrClient`` instance

        """
        from flickrsets import parsers
        parser = parsers.PersonParser(flickr_id=flickr_id, client=client)
        data = parser.formatted_data()
        obj = self.create(**data['kwargs'])
        log.info(u'Created new person: "%s"' % obj.name)
        return obj


class PhotoManager(models.Manager):
    """
    Manager of ``Photo`` model.
    """

    def get_or_create_from_api(self, flickr_id, client, **kwargs):
        """
        Returns a ``Photo`` object or creates it from Flickr API.
        
        Takes two required arguments:
        
            * ``flickr_id``: Photo's Flickr ID
            * ``client``: ``FlickrClient`` instance
            
        Returns a tuple of ``(object, created)``, where ``created`` is a
        boolean specifying wether an object was created.
        """
        try:
            return self.get(flickr_id=flickr_id), False
        except self.model.DoesNotExist:
            obj = self.create_object_from_api(
                flickr_id=flickr_id, 
                client=client, 
                **kwargs)
            return obj, True

    def create_object_from_api(self, flickr_id, client, photoset=None,
        create_tags=False, save_exif_tags=False):
        """
        Creates a new ``Photo`` object from Flickr API.
        
        Takes two required arguments:
        
            * ``flickr_id``: Photo's Flickr ID
            * ``client``: ``FlickrClient`` instance
            
        Takes three optional arguments:
        
            * ``photoset``: a ``Photoset`` object to add as foreign key
            * ``create_tags``: creates related ``Tag`` objects?
            * ``save_exif_tags``: saves photo EXIF tags
            
        """
        from flickrsets import parsers
        from flickrsets.models import Person
        from flickrsets.models import Tag
        
        parser = parsers.PhotoParser(flickr_id=flickr_id, client=client)
        data = parser.formatted_data()

        owner, created = Person.objects.get_or_create_from_api(
            flickr_id=data['fk_fields']['owner'],
            client=client)
        data['kwargs']['owner'] = owner
        log.info(u'"%s" added as owner of photo "%s"' % (
            owner.name, 
            data['kwargs']['title']))
        
        if photoset is not None:
            data['kwargs']['photoset'] = photoset
            log.debug(u'"%s" added as photoset of photo "%s"' % (
                photoset.title,
                data['kwargs']['title']))

        photo = self.create(**data['kwargs'])
        log.info(u'Created new photo: "%s"' % photo.title)

        if create_tags:
            objects, created = Tag.objects.filter_or_create_from_api(
                photo=photo,
                client=client)
                
        if save_exif_tags:
            photo.exif = parser.get_exif()
            photo.save()
        
        return photo


class PhotosetManager(models.Manager):
    """
    Manager of ``Photoset`` model.
    """
    def get_or_create_from_api(self, flickr_id, client, **kwargs):
        """
        Returns a ``Photoset`` object or creates it from Flickr API.
        
        Takes two required arguments:
        
            * ``flickr_id``: Photoset's Flickr ID
            * ``client``: ``FlickrClient`` instance
            
        Returns a tuple of ``(object, created)``, where ``created`` is a
        boolean specifying wether an object was created.
        """
        try:
            return self.get(flickr_id=flickr_id), False
        except self.model.DoesNotExist:
            obj = self.create_object_from_api(
                flickr_id=flickr_id,
                client=client,
                **kwargs)
            return obj, True

    def create_object_from_api(self, flickr_id, client, related_kwargs=None):
        """
        Creates a new ``Photoset`` object from Flickr API.
        
        Takes two required arguments:
        
            * ``flickr_id``: Photoset's Flickr ID
            * ``client``: ``FlickrClient`` instance

        Takes one optional argument:

            * ``related_kwargs``: keyword arguments passed to related objects

        """
        from flickrsets import parsers
        from flickrsets.models import Person
        from flickrsets.models import Photo
        from flickrsets.models import Photoset

        parser = parsers.PhotosetParser(flickr_id=flickr_id, client=client)
        data = parser.formatted_data()
        
        if related_kwargs is None:
            related_kwargs = {}
        
        if 'primary' not in related_kwargs:
            related_kwargs['primary'] = {}
            
        owner, created = Person.objects.get_or_create_from_api(
            flickr_id=data['fk_fields']['owner'],
            client=client)
            
        data['kwargs']['owner'] = owner
        log.info(u'"%s" added as owner of photoset "%s"' % (
            owner.name, 
            data['kwargs']['title']))

        primary, created = Photo.objects.get_or_create_from_api(
            flickr_id=data['fk_fields']['primary'],
            client=client,
            **related_kwargs['primary'])
            
        data['kwargs']['primary'] = primary
        log.info(u'"%s" added as primary of photoset "%s"' % (
            primary.title,
            data['kwargs']['title']))

        obj = Photoset.objects.create(**data['kwargs'])
        log.info(u'Created new photoset: "%s"' % obj.title)

        primary.photoset = obj
        primary.save()
        log.info(u'"%s" added as photoset of primary photo "%s"' % (
            obj.title,
            primary.title))

        return obj

    def get_photos_ids_from_api(self, flickr_id, client):
        """
        Returns Flickr IDs of photoset's photos from Flickr API.
        
        Takes two required arguments:
        
            * ``flickr_id``: Photoset's Flickr ID
            * ``client``: ``FlickrClient`` instance
            
        """
        from flickrsets import parsers
        parser = parsers.PhotosetParser(flickr_id=flickr_id, client=client)
        return parser.get_photos_ids()


class TagManager(models.Manager):
    """
    Manager of ``Tag`` model.
    """

    def filter_or_create_from_api(self, photo, client):
        """
        Returns related ``Tag`` objects of the given ``photo`` object or 
        creates them from Flickr API.
        
        Takes two required arguments:
        
            * ``photo``: ``Photo`` object
            * ``client``: ``FlickrClient`` instance
            
        Returns a tuple of ``(objects, created)``, where ``created`` is a
        boolean specifying wether objects was created.
        """
        objects = photo.tags.all()
        if objects:
            return objects, False
        else:
            objects = self.create_objects_from_api(photo=photo, client=client)
            return objects, True

    def create_objects_from_api(self, photo, client):
        """
        Creates related ``Tag`` objects of the given ``photo`` object from
        Flickr API.
        
        Takes two required arguments:
        
            * ``photo``: ``Photo`` object
            * ``client``: ``FlickrClient`` instance

        """
        from flickrsets import parsers
        from flickrsets.models import Person
        
        parser = parsers.PhotoTagsParser(
            flickr_id=photo.flickr_id, 
            client=client)
        
        objects = []
        
        for tag in parser.formatted_data():
            try:
                obj = self.get(raw=tag['kwargs']['raw'])
                objects.append(obj)
            except self.model.DoesNotExist:                
                author, created = Person.objects.get_or_create_from_api(
                    flickr_id=tag['fk_fields']['author'],
                    client=client)
                tag['kwargs']['author'] = author
                obj = self.create(**tag['kwargs'])
                objects.append(obj)
                log.info(u'Created new tag: "%s"' % obj.name)
            photo.tags.add(obj)
            log.info(u'Tagged "%s" with tag "%s"' % (photo, obj))
        
        return objects


class RegisteredSetManager(models.Manager):
    """
    Manager of ``RegisteredSet`` model.
    """

    def enabled(self):
        """
        Only returns enabled sets.
        """
        return self.filter(enabled=True)

    def disabled(self):
        """
        Only returns disabled sets.
        """
        return self.filter(enabled=False)



"""
Flickr/database synchronizer of Django Flickrsets application.
"""
import logging
import time

from flickrsets import models
from flickrsets import settings as app_settings
from flickrsets.models import Photoset
from flickrsets.models import Photo
from flickrsets.models import RegisteredSet

log = logging.getLogger('flickrsets.synchronizer')


def run(client):
    """
    Runs the synchronization process.
    """
    flush_tables()
    photosets = RegisteredSet.objects.enabled()
    photosets_ids = [photoset.flickr_id for photoset in photosets]
    photo_kwargs = {
        'create_tags': app_settings.CREATE_TAGS,
        'save_exif_tags': app_settings.SAVE_EXIF_TAGS,
    }
    for photoset_id in photosets_ids:
        photoset, created = Photoset.objects.get_or_create_from_api(
            flickr_id=photoset_id,
            client=client,
            related_kwargs={'primary': photo_kwargs})
        photos_ids = Photoset.objects.get_photos_ids_from_api(
            flickr_id=photoset_id,
            client=client)
        for photo_id in photos_ids:
            photo, created = Photo.objects.get_or_create_from_api(
                flickr_id=photo_id,
                client=client,
                photoset=photoset,
                **photo_kwargs)
            time.sleep(app_settings.SYNCHRONIZER_PHOTO_TIME_SLEEP)
        time.sleep(app_settings.SYNCHRONIZER_PHOTOSET_TIME_SLEEP)


def flush_tables():
    """
    Flush tables.
    """
    model_names = ('Person', 'Photo', 'Photoset', 'Tag')
    for model_name in model_names:
        model = getattr(models, model_name)
        table = model._meta.db_table
        model.objects.all().delete()
        log.info(u'Flushed table: %s' % table)



"""
Utils of Django Flickrsets application.
"""
import datetime
import re

import dateutil.parser
import dateutil.tz
import httplib2
from django.utils import simplejson
from django.utils.encoding import force_unicode

DEFAULT_HTTP_HEADERS = {
    "User-Agent": "Django Flickrsets/0.1",
}

DATETIME_STRING_RE = ("(?P<year>[0-9]{4})-(?P<month>[0-9]{2})-"
                      "(?P<day>[0-9]{2}) (?P<hour>[0-9]{2}):(?P<min>[0-9]{2}):"
                      "(?P<sec>[0-9]{2})")
                      

def fetch_resource(url, method='GET', body=None, username=None, password=None,
    headers=None):
    """
    Fetches and parsers a resource.
    """
    h = httplib2.Http(timeout=15)
    h.force_exception_to_status_code = True
    if username is not None or password is not None:
        h.add_credentials(username, password)
    if headers is None:
        headers = DEFAULT_HTTP_HEADERS.copy()
    response, content = h.request(url, method, body, headers)
    return content


def get_json(url, **kwargs):
    """
    Fetches and parses some JSON. Returns the deserialized JSON.
    """
    json = fetch_resource(url, **kwargs)
    return simplejson.loads(json)


def parse_date(string):
    """
    Converts a string into a (local, naive) datetime object.
    """
    if re.match(DATETIME_STRING_RE, string):
        dt = dateutil.parser.parse(string)
        if dt.tzinfo:
            dt = dt.astimezone(dateutil.tz.tzlocal()).replace(tzinfo=None)
    else:
        dt = datetime.datetime.fromtimestamp(safe_int(string))
    return dt


def safe_int(string):
    """
    Always returns an integer. Returns 0 on failure.
    """
    try:
        return int(force_unicode(string))
    except (ValueError, TypeError):
        return 0



"""
Templatetags of Django Flickrsets application.
"""
from django import template

from flickrsets import constants

register = template.Library()


class PhotoFlickrUrlsNode(template.Node):
    """
    Process class of ``do_photo_flickr_urls`` template tag.
    """

    def __init__(self, obj, varname='photo_flickr_urls'):
        """
        Initializes the template node.
        """
        self.obj = template.Variable(obj)
        self.varname = varname

    def render(self, context):
        """
        Renders the template node.
        """
        obj = self.obj.resolve(context)
        items = []
        for abbrev, rname, name in constants.FLICKR_PHOTO_URLS:
            item = {}
            item['title'] = name
            item['url'] = obj.get_image_url(size=abbrev)
            items.append(item)
        context[self.varname] = items
        return ''

# pylint: disable=W0613
def do_photo_flickr_urls(parser, token):
    """
    Sets the given ``photo`` object Flickr URLs in the context, stored in the
    template variable ``varname`` or in ``photo_flickr_urls`` variable.
    """
    bits = token.split_contents()
    if len(bits) < 1:
        raise template.TemplateSyntaxError(
            "%r tag requires at least one argument." % bits[0])
    if len(bits) == 3 and bits[2] != "as":
        raise template.TemplateSyntaxError(
            "%r tag third argument must be 'as'." % bits[0])
    if len(bits) > 4:
        raise template.TemplateSyntaxError(
            "%r tag requires takes four arguments maximum." % bits[0])
    kwargs = {}
    if len(bits) >= 2:
        kwargs['obj'] = bits[1]
    if len(bits) == 4:
        kwargs['varname'] = bits[3]
    return PhotoFlickrUrlsNode(**kwargs)


register.tag('flickrsets_get_photo_flickr_urls', do_photo_flickr_urls)



"""
Django Flickrsets application ``fsets`` command.
"""
import sys

from django.core.management.base import BaseCommand

from prettytable import PrettyTable

from flickrsets import settings as app_settings
from flickrsets import synchronizer
from flickrsets.client import FlickrClient
from flickrsets.models import RegisteredSet

COMMAND_ARGS = {
    'sync': 'Synchronizes registered Flickr sets with Flickr.',
    'list': 'Lists all registered Flickr sets.',
    'add': 'Registers a new Flickr set for synchronization.',
    'remove': 'Removes (deletes) a registered Flickr set.',
    'enable': 'Enables a disabled Flickr set.',
    'disable': 'Disables an enabled Flickr set.',
    'settings': 'Displays application settings',
}


class Command(BaseCommand):
    """
    The ``fsets`` command.
    """
    help = "Manage Flickr sets of Django Flickrsets application."

    def handle(self, *args, **options):
        """
        Handles the command.
        """
        if len(args) > 0:
            cmd = args[0]
            if cmd not in COMMAND_ARGS:
                sys.stderr.write(
                    ('fsets command only accepts these arguments: '
                     '%s\n' % ', '.join(COMMAND_ARGS.keys())),
                )
                return
            if len(args) > 1:
                cmd_arg = args[1]
                if cmd_arg == 'help':
                    sys.stderr.write('%s\n' % COMMAND_ARGS[cmd])
                    return
                else:
                    sys.stderr.write(
                        ('fsets "%s" argument only accepts "help"'
                        ' options. Other ones will be ignored.'
                        '\n' % cmd_arg),
                    )
                    return
        else:
            sys.stderr.write(
                ('fsets command accepts these arguments:'
                 ' %s\n' % ', '.join(COMMAND_ARGS.keys())),
            )
            return
        cmd = args[0]
        cmd_method = getattr(self, 'fsets_%s' % cmd)
        cmd_method()

    def fsets_sync(self):
        """
        Synchronizes Flickr sets.
        """
        client = FlickrClient(app_settings.FLICKR_API_KEY)
        synchronizer.run(client)

    def fsets_settings(self):
        """
        Displays application settings.
        """
        table = PrettyTable()
        table.field_names = ["Key", "Value"]
        table.align["Key"] = "l"
        table.align["Value"] = "l"
        settings = (
            'FLICKR_API_KEY',
            'FLICKR_USER_ID',
            'CREATE_TAGS',
            'CREATE_EXIF_TAGS',
            'EXIF_TAG_SPACE_LIST',
            'PERSON_LIST_VIEW_PAGINATE_BY',
            'PHOTO_LIST_VIEW_PAGINATE_BY',
            'PHOTOSET_LIST_VIEW_PAGINATE_BY',
            'TAG_LIST_VIEW_PAGINATE_BY',
            'SYNCHRONIZER_PHOTO_TIME_SLEEP',
            'SYNCHRONIZER_PHOTOSET_TIME_SLEEP',
        )
        for setting in settings:
            table.add_row([
                'FLICKRSETS_%s' % setting,
                getattr(app_settings, setting),
            ])
        print table.get_string()

    def fsets_list(self):
        """
        Lists all registered Flickr sets.
        """
        items = self.get_items(remote=False)
        self.print_items(items)

    def fsets_add(self):
        """
        Registers a new Flickr set.
        """
        items = self.get_items(remote=True)
        user_items = self.get_user_items("add", items)
        info = []
        for item in user_items:
            title, flickr_id = item[0], item[1]
            obj, created = RegisteredSet.objects.get_or_create(
                flickr_id=flickr_id,
                title=title)
            if created:
                print 'Added set "%s" (%s).' % (title, flickr_id)
            else:
                print 'Set "%s" (%s) already registered.' % (title, flickr_id)

    def fsets_remove(self):
        """
        Removes a Flickr set.
        """
        items = self.get_items(remote=False)
        user_items = self.get_user_items("remove", items)
        for item in user_items:
            title, flickr_id = item[0], item[1]
            RegisteredSet.objects.get(flickr_id=flickr_id).delete()
            print 'Removed set %s (%s).' % (title, flickr_id)

    def fsets_enable(self):
        """
        Enables a Flickr set.
        """
        items = self.get_items(remote=False, status_filter='disabled')
        user_items = self.get_user_items("enable", items)
        for item in user_items:
            title, flickr_id = item[0], item[1]
            obj = RegisteredSet.objects.get(flickr_id=flickr_id)
            obj.enabled = True
            obj.save()
            print 'Set %s (%s) is enabled.' % (title, flickr_id)

    def fsets_disable(self):
        """
        Disables a Flickr set.
        """
        items = self.get_items(remote=False, status_filter='enabled')
        user_items = self.get_user_items("disable", items)
        for item in user_items:
            title, flickr_id = item[0], item[1]
            obj = RegisteredSet.objects.get(flickr_id=flickr_id)
            obj.enabled = False
            obj.save()
            print 'Set %s (%s) is disabled.' % (title, flickr_id)

    def print_items(self, items):
        """
        Prints items
        """
        table = PrettyTable()
        table.field_names = ["ID", "Title", "Flickr ID", "Status"]
        table.align["ID"] = "r"
        table.align["Title"] = "l"
        table.align["Flickr ID"] = "r"
        table.align["Status"] = "c"
        for idx, title, flickr_id, status in items:
            table.add_row([str(idx), title, flickr_id, status])
        print table.get_string()

    def get_remote_items(self):
        """
        Returns set list from Flickr API.
        """
        client = FlickrClient(app_settings.FLICKR_API_KEY)
        json = client.photosets.getList(user_id=app_settings.FLICKR_USER_ID)
        items = json.get('photosets').get('photoset')
        return [
            (idx, item.get('title').get('_content'), item.get('id'), "REMOTE")
            for idx, item in enumerate(items)]

    def get_db_items(self, status_filter=None):
        """
        Returns set list from database.
        """
        items = RegisteredSet.objects.all()
        if status_filter == 'enabled':
            items = RegisteredSet.objects.enabled()
        if status_filter == 'disabled':
            items = RegisteredSet.objects.disabled()

        def status_msg(boolean):
            if boolean:
                return "ENABLED"
            return "DISABLED"

        return [
            (idx, item.title, item.flickr_id, status_msg(item.enabled))
            for idx, item in enumerate(items)]

    def get_items(self, remote=False, status_filter=None):
        """
        Returns a list of sets from Flickr API or database.
        """
        if remote:
            items = self.get_remote_items()
        else:
            items = self.get_db_items(status_filter)
        if not items:
            print 'No set available.'
            sys.exit()
        return items

    def get_user_items(self, verb, items):
        """
        Displays the ``verb`` in the question sent to the user and returns a
        list of tuples containing items title and Flickr ID, by extracting
        them from ``items``.

        Example of returned value::

            [('Super Set', '19879872'), ('Other Set', '2890820398')]

        """
        self.print_items(items)
        try:
            choices = raw_input(
                ("\nWhich Flickr set(s) you want to %s? " % str(verb)))
        except KeyboardInterrupt:
            sys.exit()
        if not choices:
            sys.exit()
        choices = choices.replace(',', '').split(' ')
        try:
            int_choices = []
            for choice in choices:
                int_choices.append(int(choice))
        except ValueError:
            sys.exit()
        user_items = []
        for choice in int_choices:
            user_items.append((items[choice][1], items[choice][2]))
        return user_items



"""
URLs of Django Flickrsets application related to ``Photoset`` model.
"""
from django.conf.urls.defaults import patterns
from django.conf.urls.defaults import url


urlpatterns = patterns('flickrsets.views',
    url(r'^(?P<flickr_id>[-\w]+)/$',
        view='photoset_photo_list',
        name='flickrsets-photoset',
    ),
    url(r'^$',
        view='photoset_list',
        name='flickrsets-photosets',
    ),
)



"""
Views tests of Django Flickrsets application.
"""
from django import test
from django.core.urlresolvers import reverse

from flickrsets.models import Person
from flickrsets.models import Photo
from flickrsets.models import Photoset
from flickrsets.models import Tag


class ViewsTest(test.TestCase):
    """
    Views tests.
    """
    fixtures = ['flickrsets/tests']

    def test_person_list(self):
        """
        Tests ``person_list`` view.
        """
        response = self.client.get(reverse('flickrsets-people'))
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'flickrsets/person/list.html')

    def test_person_photo_list(self):
        """
        Tests ``person_photo_list`` view.
        """
        people = Person.objects.all()
        for person in people:
            response = self.client.get(
                reverse('flickrsets-person', args=[person.flickr_id]))
            self.assertEquals(response.status_code, 200)
            self.assertTemplateUsed(response, 'flickrsets/person/photo_list.html')

    def test_person_photo_list_404(self):
        """
        Tests 404 on ``person_photo_list`` view
        """
        response = self.client.get(
            reverse('flickrsets-person', args=['unknown']))
        self.assertEquals(response.status_code, 404)

    def test_photo_list(self):
        """
        Tests ``photo_list`` view.
        """
        response = self.client.get(reverse('flickrsets-photos'))
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'flickrsets/photo/list.html')

    def test_photo_detail(self):
        """
        Tests ``photo_detail`` view.
        """
        photos = Photo.objects.all()
        for photo in photos:
            response = self.client.get(
                reverse('flickrsets-photo', args=[photo.flickr_id]))
            self.assertEquals(response.status_code, 200)
            self.assertTemplateUsed(response, 'flickrsets/photo/detail.html')

    def test_photo_detail_404(self):
        """
        Tests 404 on ``photo_detail`` view.
        """
        response = self.client.get(
            reverse('flickrsets-photo', args=['unknown']))
        self.assertEquals(response.status_code, 404)

    def test_photoset_list(self):
        """
        Tests ``photoset_list`` view.
        """
        response = self.client.get(reverse('flickrsets-photosets'))
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'flickrsets/photoset/list.html')

    def test_photoset_photo_list(self):
        """
        Tests ``photoset_photo_list`` view.
        """
        photosets = Photoset.objects.all()
        for photoset in photosets:
            response = self.client.get(
                reverse('flickrsets-photoset', args=[photoset.flickr_id]))
            self.assertEquals(response.status_code, 200)
            self.assertTemplateUsed(response, 'flickrsets/photoset/photo_list.html')

    def test_photoset_photo_list_404(self):
        """
        Tests 404 on ``photoset_photo_list`` view
        """
        response = self.client.get(
            reverse('flickrsets-photoset', args=['unknown']))
        self.assertEquals(response.status_code, 404)

    def test_tag_list(self):
        """
        Tests ``tag_list`` view.
        """
        response = self.client.get(reverse('flickrsets-tags'))
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'flickrsets/tag/list.html')

    def test_tag_photo_list(self):
        """
        Tests ``tag_photo_list`` view.
        """
        tags = Tag.objects.all()
        for tag in tags:
            response = self.client.get(
                reverse('flickrsets-tag', args=[tag.name]))
            self.assertEquals(response.status_code, 200)
            self.assertTemplateUsed(response, 'flickrsets/tag/photo_list.html')

    def test_tag_photo_list_404(self):
        """
        Tests 404 on ``tag_photo_list`` view.
        """
        response = self.client.get(
            reverse('flickrsets-tag', args=['unknown']))
        self.assertEquals(response.status_code, 404)



# -*- coding: utf-8
"""
Constants of Django Flickrsets application.
"""
from django.utils.translation import ugettext_lazy as _


# Prefix used for application's tables
APP_TABLE_PREFIX = 'flickrsets'

# Flickr's photo URLs
FLICKR_PHOTO_URL_SQUARE = 'sq'
FLICKR_PHOTO_URL_THUMBNAIL = 't'
FLICKR_PHOTO_URL_SMALL = 's'
FLICKR_PHOTO_URL_MEDIUM = 'm'
FLICKR_PHOTO_URL_LARGE = 'b'
FLICKR_PHOTO_URL_ORIGINAL = 'o'
FLICKR_PHOTO_URLS = (
    (FLICKR_PHOTO_URL_SQUARE, 'square', _('Square')),
    (FLICKR_PHOTO_URL_THUMBNAIL, 'thumbnail', _('Thumbnail')),
    (FLICKR_PHOTO_URL_SMALL, 'small', _('Small')),
    (FLICKR_PHOTO_URL_MEDIUM, 'medium', _('Medium')),
    (FLICKR_PHOTO_URL_LARGE, 'large', _('Large')),
    (FLICKR_PHOTO_URL_ORIGINAL, 'original', _('Original')))

# Flickr's photo URLs
FLICKR_PHOTO_SOURCE_SQUARE = 's'
FLICKR_PHOTO_SOURCE_THUMBNAIL = 't'
FLICKR_PHOTO_SOURCE_SMALL = 'm'
FLICKR_PHOTO_SOURCE_MEDIUM = 'default'
FLICKR_PHOTO_SOURCE_LARGE = 'b'
FLICKR_PHOTO_SOURCE_ORIGINAL = 'o'
FLICKR_PHOTO_SOURCES = (
    (FLICKR_PHOTO_SOURCE_SQUARE, 'square', _('Square')),
    (FLICKR_PHOTO_SOURCE_THUMBNAIL, 'thumbnail', _('Thumbnail')),
    (FLICKR_PHOTO_SOURCE_SMALL, 'small', _('Small')),
    (FLICKR_PHOTO_SOURCE_MEDIUM, 'medium', _('Medium')),
    (FLICKR_PHOTO_SOURCE_LARGE, 'large', _('Large')),
    (FLICKR_PHOTO_SOURCE_ORIGINAL, 'original', _('Original')))

# Flickr's genders
FLICKR_GENDER_FEMALE = 'F'
FLICKR_GENDER_MALE = 'M'
FLICKR_GENDER_OTHER = 'O'
FLICKR_GENDER_RATHER_NOT_SAY = 'X'
FLICKR_DEFAULT_GENDER = 'X'
FLICKR_GENDERS = (
    (FLICKR_GENDER_FEMALE, _('Female')),
    (FLICKR_GENDER_MALE, _('Male')),
    (FLICKR_GENDER_OTHER, _('Other')),
    (FLICKR_GENDER_RATHER_NOT_SAY, _('Rather not say')))

# Flickr's licenses
FLICKR_LICENSES = (
    (0, 'All Rights Reserved'),
    (1, 'Attribution-NonCommercial-ShareAlike License'),
    (2, 'Attribution-NonCommercial License'),
    (3, 'Attribution-NonCommercial-NoDerivs License'),
    (4, 'Attribution License'),
    (5, 'Attribution-ShareAlike License'),
    (6, 'Attribution-NoDerivs License'))

# Flickr's media
FLICKR_MEDIAS = (
    ('photo', _('Photo')),
    ('video', _('Video')),
    ('other', _('Other')))

# Test
TEST_PERSON_FLICKR_ID = u'94238521@N00'
TEST_PHOTO_FLICKR_ID = u'4242733600'
TEST_PHOTOSET_FLICKR_ID = u'72157623007721343'
TEST_TAG_RAW = u'rock'



"""
Tests parsers of Django Flickrsets application.
"""
import unittest

from flickrsets import constants
from flickrsets.parsers import PersonParser
from flickrsets.parsers import PhotoParser
from flickrsets.parsers import PhotoTagsParser
from flickrsets.parsers import PhotosetParser
from flickrsets.tests.client import FakeClient


class PersonParserTest(unittest.TestCase):
    """
    Tests ``PersonParser`` parser.
    """
    FLICKR_ID = constants.TEST_PERSON_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_raw_data(self):
        """
        Tests ``raw_data`` returned value.
        """
        parser = PersonParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        data = parser.raw_data()
        self.assertEquals(data.get('nsid'), self.FLICKR_ID)

    def test_formatted_data(self):
        """
        Tests ``formatted_data`` returned value.
        """
        parser = PersonParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        data = parser.formatted_data()
        self.assertEquals(len(data), 1)
        self.assertEquals(data['kwargs']['flickr_id'], self.FLICKR_ID)
        self.assertEquals('fk_fields' in data, False)


class PhotoParserTest(unittest.TestCase):
    """
    Tests ``PhotoParser`` parser.
    """
    FLICKR_ID = constants.TEST_PHOTO_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_raw_data(self):
        """
        Tests ``raw_data`` returned value.
        """
        parser = PhotoParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        data = parser.raw_data()
        self.assertEquals(data['id'], self.FLICKR_ID)

    def test_formatted_data(self):
        """
        Tests ``formatted_data`` value.
        """
        parser = PhotoParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        data = parser.formatted_data()
        self.assertEquals(len(data), 2)
        self.assertEquals(data['kwargs']['flickr_id'], self.FLICKR_ID)
        self.assertEquals('fk_fields' in data, True)
        self.assertEquals(len(data['fk_fields']), 1)
        self.assertEquals('owner' in data['fk_fields'], True)


class PhotoTagsParserTest(unittest.TestCase):
    """
    Tests ``PhotoTagsParser`` parser.
    """
    FLICKR_ID = constants.TEST_PHOTO_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_raw_data(self):
        """
        Tests ``raw_data`` returned value.
        """
        parser = PhotoTagsParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        data = parser.raw_data()
        self.assertEquals(len(data), 3)

    def test_formatted_data(self):
        """
        Tests ``formatted_data`` returned value.
        """
        parser = PhotoTagsParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)   
        data = parser.formatted_data()
        self.assertEquals(len(data), 3)


class PhotosetParserTest(unittest.TestCase):
    """
    Tests ``PhotosetParser`` parser.
    """
    FLICKR_ID = constants.TEST_PHOTOSET_FLICKR_ID
    api_client = FakeClient('apikey')

    def test_raw_data(self):
        """
        Tests ``raw_data`` returned value.
        """
        parser = PhotosetParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        data = parser.raw_data()
        self.assertEquals(data['id'], self.FLICKR_ID)

    def test_formatted_data(self):
        """
        Tests ``formatted_data`` returned value.
        """
        parser = PhotosetParser(
            flickr_id=self.FLICKR_ID, 
            client=self.api_client)
        data = parser.formatted_data()
        self.assertEquals(len(data), 2)
        self.assertEquals(data['kwargs']['flickr_id'], self.FLICKR_ID)
        self.assertEquals('fk_fields' in data, True)
        self.assertEquals(len(data['fk_fields']), 2)
        self.assertEquals('owner' in data['fk_fields'], True)
        self.assertEquals('primary' in data['fk_fields'], True)



"""
Flickr client of Django Flickrsets application.

Client's code is borrowed from Jacob Kaplan-Moss Jellyroll project
(http://github.com/jacobian/jellyroll).
"""
import urllib

from flickrsets import utils


class FlickrError(Exception):
    """
    A Flickr Error Exception.
    """

    def __init__(self, code, message):
        """
        Initializes exception.
        """
        self.code = code
        self.message = message

    def __str__(self):
        """
        Exception human readable string.
        """
        return u'FlickrError %s: %s' % (self.code, self.message)


class FlickrClient(object):
    """
    The Flickr Client.
    """

    def __init__(self, api_key, method='flickr'):
        """
        Initializes the Flickr client.
        """
        self.api_key = api_key
        self.method = method

    def __getattr__(self, method):
        """
        Catches Flickr API methods.
        """
        return FlickrClient(self.api_key, '%s.%s' % (self.method, method))

    def __repr__(self):
        """
        Object's representation.
        """
        return '<FlickrClient: %s>' % self.method

    def __call__(self, **params):
        """
        Maps Flickr API methods.
        """
        params['method'] = self.method
        params['api_key'] = self.api_key
        params['format'] = 'json'
        params['nojsoncallback'] = '1'
        url = 'http://flickr.com/services/rest/?' + urllib.urlencode(params)
        json = utils.get_json(url)
        if json.get('stat', '') == 'fail':
            raise FlickrError(json['code'], json['message'])
        return json



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'RegisteredSet'
        db.create_table('flickrsets_registered_set', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('flickr_id', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
        ))
        db.send_create_signal('flickrsets', ['RegisteredSet'])

        # Adding model 'Person'
        db.create_table('flickrsets_person', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('flickr_id', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('username', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('realname', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('photos_url', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('profile_url', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('mobile_url', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('icon_server', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('icon_farm', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('gender', self.gf('django.db.models.fields.CharField')(default='X', max_length=1, blank=True)),
            ('location', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('photos_count', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal('flickrsets', ['Person'])

        # Adding model 'Photoset'
        db.create_table('flickrsets_photoset', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('owner', self.gf('django.db.models.fields.related.ForeignKey')(related_name='photosets', to=orm['flickrsets.Person'])),
            ('primary', self.gf('django.db.models.fields.related.ForeignKey')(related_name='photoset_primaries', to=orm['flickrsets.Photo'])),
            ('flickr_id', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('server', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('farm', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('photos_count', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal('flickrsets', ['Photoset'])

        # Adding model 'Tag'
        db.create_table('flickrsets_photo_tag', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(related_name='tags', to=orm['flickrsets.Person'])),
            ('flickr_id', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('raw', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('flickrsets', ['Tag'])

        # Adding model 'Photo'
        db.create_table('flickrsets_photo', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('photoset', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='photos', null=True, to=orm['flickrsets.Photoset'])),
            ('owner', self.gf('django.db.models.fields.related.ForeignKey')(related_name='photos', to=orm['flickrsets.Person'])),
            ('flickr_id', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('server', self.gf('django.db.models.fields.IntegerField')()),
            ('farm', self.gf('django.db.models.fields.IntegerField')()),
            ('license', self.gf('django.db.models.fields.SmallIntegerField')(null=True, blank=True)),
            ('media', self.gf('django.db.models.fields.CharField')(max_length=10, null=True, blank=True)),
            ('secret', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('original_secret', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('original_format', self.gf('django.db.models.fields.CharField')(max_length=10, null=True, blank=True)),
            ('views_count', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('comments_count', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('date_uploaded', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('date_posted', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('date_taken', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal('flickrsets', ['Photo'])

        # Adding M2M table for field tags on 'Photo'
        db.create_table('flickrsets_photo_tags', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('photo', models.ForeignKey(orm['flickrsets.photo'], null=False)),
            ('tag', models.ForeignKey(orm['flickrsets.tag'], null=False))
        ))
        db.create_unique('flickrsets_photo_tags', ['photo_id', 'tag_id'])

        # Adding model 'ExifTag'
        db.create_table('flickrsets_exif_tag', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('photo', self.gf('django.db.models.fields.related.ForeignKey')(related_name='exif_tags', to=orm['flickrsets.Photo'])),
            ('tag_space', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('tag_space_id', self.gf('django.db.models.fields.IntegerField')()),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('raw', self.gf('django.db.models.fields.TextField')()),
            ('clean', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('flickrsets', ['ExifTag'])


    def backwards(self, orm):
        
        # Deleting model 'RegisteredSet'
        db.delete_table('flickrsets_registered_set')

        # Deleting model 'Person'
        db.delete_table('flickrsets_person')

        # Deleting model 'Photoset'
        db.delete_table('flickrsets_photoset')

        # Deleting model 'Tag'
        db.delete_table('flickrsets_photo_tag')

        # Deleting model 'Photo'
        db.delete_table('flickrsets_photo')

        # Removing M2M table for field tags on 'Photo'
        db.delete_table('flickrsets_photo_tags')

        # Deleting model 'ExifTag'
        db.delete_table('flickrsets_exif_tag')


    models = {
        'flickrsets.exiftag': {
            'Meta': {'object_name': 'ExifTag', 'db_table': "'flickrsets_exif_tag'"},
            'clean': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'photo': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'exif_tags'", 'to': "orm['flickrsets.Photo']"}),
            'raw': ('django.db.models.fields.TextField', [], {}),
            'tag_space': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'tag_space_id': ('django.db.models.fields.IntegerField', [], {})
        },
        'flickrsets.person': {
            'Meta': {'object_name': 'Person'},
            'flickr_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'gender': ('django.db.models.fields.CharField', [], {'default': "'X'", 'max_length': '1', 'blank': 'True'}),
            'icon_farm': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'icon_server': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'mobile_url': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'photos_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'photos_url': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'profile_url': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'realname': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        'flickrsets.photo': {
            'Meta': {'object_name': 'Photo'},
            'comments_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'date_posted': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_taken': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_uploaded': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'farm': ('django.db.models.fields.IntegerField', [], {}),
            'flickr_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'license': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'media': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'original_format': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'original_secret': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'photos'", 'to': "orm['flickrsets.Person']"}),
            'photoset': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'photos'", 'null': 'True', 'to': "orm['flickrsets.Photoset']"}),
            'secret': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'server': ('django.db.models.fields.IntegerField', [], {}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'photos'", 'null': 'True', 'symmetrical': 'False', 'to': "orm['flickrsets.Tag']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'views_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'flickrsets.photoset': {
            'Meta': {'object_name': 'Photoset'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'farm': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'flickr_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'photosets'", 'to': "orm['flickrsets.Person']"}),
            'photos_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'primary': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'photoset_primaries'", 'to': "orm['flickrsets.Photo']"}),
            'server': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'flickrsets.registeredset': {
            'Meta': {'object_name': 'RegisteredSet', 'db_table': "'flickrsets_registered_set'"},
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'flickr_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'flickrsets.tag': {
            'Meta': {'object_name': 'Tag', 'db_table': "'flickrsets_photo_tag'"},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'tags'", 'to': "orm['flickrsets.Person']"}),
            'flickr_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'raw': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['flickrsets']



"""
Flickr parser classes of Django Flickrsets application.
"""
import logging

from django.utils.encoding import smart_unicode

from flickrsets import models
from flickrsets import settings as app_settings

log = logging.getLogger('flickrsets.parsers')


# Base classes
# -----------------------------------------------------------------------------
class BaseParser(object):
    """
    Parser base class.
    """
    model = None
    fields_map = None
    fk_fields_map = None

    def __init__(self, flickr_id=None, client=None):
        """
        Parser initialization.
        """
        self.flickr_id = flickr_id
        self.client = client

    def raw_data(self):
        """
        Returns data to format returned by Flickr API.
        """
        pass
        
    def formatted_data(self):
        """
        Returns data of ``raw_data()`` formatted for object creation.
        """
        return self.format_raw_data()

    def format_raw_data(self):
        """
        Formats data of ``raw_data()`` for object creation.
        """
        raw_data = self.raw_data()
        is_list = True
        if not isinstance(raw_data, list):
            is_list = False
            raw_data = [raw_data]
        items = []
        for item in raw_data:
            formatted_data = {}
            if self.fields_map:
                formatted_data['kwargs'] = self.format_data_for_fields(
                    self.fields_map,
                    item)
            if self.fk_fields_map:
                formatted_data['fk_fields'] = self.format_data_for_fk_fields(
                    self.fk_fields_map,
                    item)
            items.append(formatted_data)
        if not is_list:
            items = items[0]
        return items

    def format_data_for_fields(self, fields_map, raw_data, model=None):
        """
        Formats data of ``fields_map`` fields.
        """
        formatted_data = {}
        if model is None:
            model = self.model
        for field_name, api_name in fields_map:
            field = model._meta.get_field_by_name(field_name)[0]
            value = self.get_raw_value(api_name, raw_data)
            field_value = self.get_safe_value_for_field(field, value)
            if field_value is not None:
                formatted_data[field_name] = field_value
        return formatted_data

    def format_data_for_fk_fields(self, fields_map, raw_data):
        """
        Formats data of ``fk_fields_map`` fields.
        """
        formatted_data = {}
        for field_name, api_name in fields_map:
            value = self.get_raw_value(api_name, raw_data)
            field_value = self.get_safe_value(value, 'str')
            formatted_data[field_name] = field_value
        return formatted_data

    def get_raw_value(self, key, dictionary):
        """
        Returns the raw value of the given ``key`` in the given ``dictionary``.
        """
        if dictionary is not None:
            keys = key.split('.')
            if len(keys) == 2:
                return dictionary.get(keys[0]).get(keys[1])
            return dictionary.get(key)

    def get_safe_value_for_field(self, field, value):
        """
        Returns safe value for ``value`` for a given ``field``.
        """
        from django.db.models import fields
        is_int = isinstance(field, fields.IntegerField)
        is_char = isinstance(field, fields.CharField)
        is_text = isinstance(field, fields.TextField)
        is_date = isinstance(field, fields.DateField)
        is_datetime = isinstance(field, fields.DateTimeField)
        if is_char or is_text:
            return self.get_safe_value(value, 'str')
        if is_int:
            return self.get_safe_value(value, 'int')
        if is_date or is_datetime:
            return self.get_safe_value(value, 'date')

    def get_safe_value(self, value, kind='str'):
        """
        Returns safe value for ``value`` for a given ``kind``.
        """
        from flickrsets import utils
        if value is not None:
            if isinstance(value, dict):
                return value.get('_content', None)
            if kind == 'str':
                return smart_unicode(value)
            if kind == 'int':
                return utils.safe_int(value)
            if kind == 'date':
                return utils.parse_date(value)


# Models parsers
# -----------------------------------------------------------------------------
class PersonParser(BaseParser):
    """
    Parser mapped on ``Person`` model which returns data for a given person
    Flickr ID.
    """
    model = models.Person
    
    fields_map = (
        ('flickr_id', 'nsid'),
        ('username', 'username'),
        ('realname', 'realname'),
        ('photos_url', 'photosurl'),
        ('profile_url', 'profileurl'),
        ('mobile_url', 'mobileurl'),
        ('icon_server', 'iconserver'),
        ('icon_farm', 'iconfarm'),
        ('gender', 'gender'),
        ('location', 'location'),
        ('photos_count', 'photos.count'))
    
    def raw_data(self):
        """
        Returns data returned by ``flickr.people.getInfo``.
        """
        log.info(u'PersonParser -- flickr.people.getInfo()')
        json = self.client.people.getInfo(user_id=self.flickr_id)
        return json.get('person')


class PhotoParser(BaseParser):
    """
    Parser mapped on ``Photo`` model which returns data for a given photo
    Flickr ID.
    """
    model = models.Photo
    
    fields_map = (
        ('flickr_id', 'id'),
        ('title', 'title'),
        ('description', 'description'),
        ('server', 'server'),
        ('farm', 'farm'),
        ('license', 'license'),
        ('media', 'media'),
        ('secret', 'secret'),
        ('original_secret', 'originalsecret'),
        ('original_format', 'originalformat'),
        ('views_count', 'views'),
        ('comments_count', 'comments'),
        ('date_uploaded', 'dateuploaded'),
        ('date_posted', 'dates.posted'),
        ('date_updated', 'dates.lastupdate'),
        ('date_taken', 'dates.taken'))
        
    fk_fields_map = (
        ('owner', 'owner.nsid'),)

    def raw_data(self):
        """
        Returns data returned by ``flickr.photos.getInfo``.
        """
        log.info(u'PhotoParser -- flickr.photos.getInfo()')
        json = self.client.photos.getInfo(photo_id=self.flickr_id)
        return json.get('photo')

    def get_exif(self):
        """
        Returns data returned by ``flickr.photos.getExif``.
        """
        log.info(u'PhotoExifTagsParser -- flickr.photos.getExif()')
        json = self.client.photos.getExif(photo_id=self.flickr_id)
        tags = json['photo']['exif']
        spaces = app_settings.EXIF_TAG_SPACE_LIST
        tags = [tag for tag in tags if tag['tagspace'] in spaces]
        return self._convert_exif(tags)
        
    def _convert_exif(self, exif):
        """
        Converts EXIF data.
        """
        converted = {}
        for e in exif:
            key = smart_unicode(e['label'])
            val = e.get('clean', e['raw'])['_content']
            val = smart_unicode(val)
            converted[key] = val
        return converted


class PhotosetParser(BaseParser):
    """
    Parser mapped on ``Photoset`` model which returns data for a given photoset
    Flickr ID.
    """
    model = models.Photoset
    
    fields_map = (
        ('flickr_id', 'id'),
        ('title', 'title'),
        ('description', 'description'),
        ('server', 'server'),
        ('farm', 'farm'),
        ('photos_count', 'photos'))
        
    fk_fields_map = (
        ('owner', 'owner'),
        ('primary', 'primary'))

    def raw_data(self):
        """
        Returns data returned by ``flickr.photosets.getInfo``.
        """
        log.info(u'PhotosetParser -- flickr.photosets.getInfo()')
        json = self.client.photosets.getInfo(
            photoset_id=self.flickr_id)
        return json.get('photoset')

    def get_photos_ids(self):
        """
        Returns a list of photos IDs of the photoset.
        Calls ``flickr.photosets.getPhotos``.
        """
        log.info(u'PhotosetParser -- flickr.photosets.getPhotos()')
        json = self.client.photosets.getPhotos(
            photoset_id=self.flickr_id)
        photos = json.get('photoset').get('photo')
        return [self.get_safe_value(p.get('id'), 'str') for p in photos]


class PhotoTagsParser(BaseParser):
    """
    Parser mapped on ``Tag`` model which returns data for related ``Tag``
    objects of a given photo Flickr ID.
    """
    model = models.Tag
    
    fields_map = (
        ('flickr_id', 'id'),
        ('name', '_content'),
        ('raw', 'raw'))
        
    fk_fields_map = (
        ('author', 'author'),)

    def raw_data(self):
        """
        Returns data returned by ``flickr.tags.getListPhoto``.
        """
        log.info(u'PhotoTagParser -- flickr.tags.getListPhoto')
        json = self.client.tags.getListPhoto(photo_id=self.flickr_id)
        return json.get('photo').get('tags').get('tag')



"""
URLs of Django Flickrsets application related to ``Tag`` model.
"""
from django.conf.urls.defaults import patterns
from django.conf.urls.defaults import url


urlpatterns = patterns('flickrsets.views',
    url(r'^(?P<name>[-\w]+)/$',
        view='tag_photo_list',
        name='flickrsets-tag',
    ),
    url(r'^$',
        view='tag_list',
        name='flickrsets-tags',
    ),
)



"""
Models of Django Flickrsets application.
"""
from django.db import models
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

from flickrsets import constants
from flickrsets.managers import PersonManager
from flickrsets.managers import PhotoManager
from flickrsets.managers import PhotosetManager
from flickrsets.managers import RegisteredSetManager
from flickrsets.managers import TagManager


class RegisteredSet(models.Model):
    """
    A registered Flickr Set.
    """
    flickr_id = models.CharField(
        verbose_name=_('Flickr ID'),
        max_length=255,
        unique=True)

    title = models.CharField(
        verbose_name=_('title'),
        max_length=255)

    enabled = models.BooleanField(
        verbose_name=_('enabled'),
        default=True)

    objects = RegisteredSetManager()

    class Meta:
        """
        Model metadata.
        """
        db_table = '%s_registered_set' % constants.APP_TABLE_PREFIX
        verbose_name = _('registered set')
        verbose_name_plural = _('registered sets')

    def __unicode__(self):
        """
        Object's human readable unicode string.
        """
        return u'%s - %s' % (self.title, self.flickr_id)


class Person(models.Model):
    """
    A Flickr Person (User).
    """
    flickr_id = models.CharField(
        verbose_name=_('flickr ID'),
        max_length=255)

    username = models.CharField(
        verbose_name=_('username'),
        max_length=255,
        null=True,
        blank=True)

    realname = models.CharField(
        verbose_name=_('realname'),
        max_length=255,
        null=True,
        blank=True)

    photos_url = models.CharField(
        verbose_name=_('photos URL'),
        max_length=255)

    profile_url = models.CharField(
        verbose_name=_('profile URL'),
        max_length=255)

    mobile_url = models.CharField(
        verbose_name=_('mobile URL'),
        max_length=255)

    icon_server = models.IntegerField(
        verbose_name=_('icon server'),
        null=True,
        blank=True)

    icon_farm = models.IntegerField(
        verbose_name=_('icon farm'),
        null=True,
        blank=True)

    gender = models.CharField(
        verbose_name=_('gender'),
        max_length=1,
        choices=constants.FLICKR_GENDERS,
        default=constants.FLICKR_DEFAULT_GENDER,
        blank=True)

    location = models.CharField(
        verbose_name=_('location'),
        max_length=255,
        null=True,
        blank=True)

    photos_count = models.IntegerField(
        verbose_name=_('photos count'),
        null=True,
        blank=True)

    objects = PersonManager()

    class Meta:
        """
        Model metadata.
        """
        db_table = '%s_person' % constants.APP_TABLE_PREFIX
        ordering = ('flickr_id',)
        verbose_name = _('person')
        verbose_name_plural = _('persons')

    def __unicode__(self):
        """
        Object's human readable unicode string.
        """
        return u'%s' % self.name

    @models.permalink
    def get_absolute_url(self):
        """
        Returns object's absolute URL.
        """
        return ('flickrsets-person', (), {'flickr_id': self.flickr_id})

    @property
    def name(self):
        """
        Returns person name.
        """
        return self.realname or self.username or self.flickr_id

    @property
    def buddy_icon_url(self):
        """
        Returns person's buddy icon URL.
        """
        if self.icon_farm and self.icon_server:
            return u'http://farm%s.static.flickr.com/%s/buddyicons/%s.jpg' % (
                self.icon_farm,
                self.icon_server,
                self.flickr_id)
        return u'http://www.flickr.com/images/buddyicon.jpg'


class Photoset(models.Model):
    """
    A Flickr Photoset.
    """
    owner = models.ForeignKey(
        'flickrsets.Person',
        verbose_name=_('owner'),
        related_name='photosets')

    primary = models.ForeignKey(
        'flickrsets.Photo',
        verbose_name=_('primary'),
        related_name='photoset_primaries')

    flickr_id = models.CharField(
        verbose_name=_('flickr ID'),
        max_length=255)

    title = models.CharField(
        verbose_name=_('title'),
        max_length=255)

    description = models.TextField(
        verbose_name=_('description'),
        null=True,
        blank=True)

    server = models.IntegerField(
        verbose_name=_('server'),
        null=True,
        blank=True)

    farm = models.IntegerField(
        verbose_name=_('farm'),
        null=True,
        blank=True)

    photos_count = models.IntegerField(
        verbose_name=_('photos'),
        null=True,
        blank=True)

    objects = PhotosetManager()

    class Meta:
        """
        Model metadata.
        """
        db_table = '%s_photoset' % constants.APP_TABLE_PREFIX
        ordering = ('title',)
        verbose_name = _('photoset')
        verbose_name_plural = _('photosets')

    def __unicode__(self):
        """
        Object's human readable unicode string.
        """
        return u'%s' % self.title

    @models.permalink
    def get_absolute_url(self):
        """
        Returns object's absolute URL.
        """
        return ('flickrsets-photoset', (), {'flickr_id': self.flickr_id})

    def get_flickr_url(self):
        """
        Returns object's Flickr URL.
        """
        return u'http://flickr.com/photos/%s/sets/%s/' % (
            self.owner.photos_url,
            self.flickr_id)


class Photo(models.Model):
    """
    A Flickr Photo.
    """
    photoset = models.ForeignKey(
        'flickrsets.Photoset',
        verbose_name=_('photoset'),
        related_name='photos',
        null=True,
        blank=True)

    owner = models.ForeignKey(
        'flickrsets.Person',
        verbose_name=_('owner'),
        related_name='photos')

    tags = models.ManyToManyField(
        'flickrsets.Tag',
        verbose_name=_('tags'),
        null=True,
        blank=True,
        related_name='photos')

    flickr_id = models.CharField(
        verbose_name=_('flickr ID'),
        max_length=255)

    title = models.CharField(
        verbose_name=_('title'),
        max_length=255)

    description = models.TextField(
        verbose_name=_('description'),
        null=True,
        blank=True)

    server = models.IntegerField(
        verbose_name=_('server'))

    farm = models.IntegerField(
        verbose_name=_('farm'))

    license = models.SmallIntegerField(
        verbose_name=_('license'),
        choices=constants.FLICKR_LICENSES,
        null=True,
        blank=True)

    media = models.CharField(
        verbose_name=_('media'),
        max_length=10,
        choices=constants.FLICKR_MEDIAS,
        null=True,
        blank=True)

    secret = models.CharField(
        verbose_name=_('secret'),
        max_length=255)

    original_secret = models.CharField(
        verbose_name=_('original secret'),
        max_length=255,
        null=True,
        blank=True)

    original_format = models.CharField(
        verbose_name=_('original format'),
        max_length=10,
        null=True,
        blank=True)

    views_count = models.IntegerField(
        verbose_name=_('views'),
        null=True,
        blank=True)

    comments_count = models.IntegerField(
        verbose_name=_('comments'),
        null=True,
        blank=True)

    date_uploaded = models.DateTimeField(
        verbose_name=_('date uploaded'),
        null=True,
        blank=True)

    date_posted = models.DateTimeField(
        verbose_name=_('date posted'),
        null=True,
        blank=True)

    date_updated = models.DateTimeField(
        verbose_name=_('date updated'),
        null=True,
        blank=True)

    date_taken = models.DateTimeField(
        verbose_name=_('date taken'),
        null=True,
        blank=True)
        
    _exif = models.TextField(
        verbose_name=_('EXIF'),
        null=True,
        blank=True)

    objects = PhotoManager()

    class Meta:
        """
        Model metadata.
        """
        db_table = '%s_photo' % constants.APP_TABLE_PREFIX
        ordering = ('title',)
        verbose_name = _('photo')
        verbose_name_plural = _('photos')

    def __unicode__(self):
        """
        Object's human readable string.
        """
        return u'%s' % self.title

    def __getattr__(self, name):
        """
        Overrides ``__getattr__`` to adds dynamic properties.
        """
        # image_VERSION_url properties
        for size, size_rname, size_name in constants.FLICKR_PHOTO_URLS:
            if name in ('image_url', 'image_medium_url'):
                medium_size = constants.FLICKR_PHOTO_URL_MEDIUM
                return self.get_image_url(size=medium_size)
            if name == 'image_%s_url' % size_rname:
                return self.get_image_url(size=size)
        # image_VERSION_source properties
        for size, size_rname, size_name in constants.FLICKR_PHOTO_SOURCES:
            if name in ('image_source', 'image_medium_source'):
                medium_size = constants.FLICKR_PHOTO_SOURCE_MEDIUM
                return self.get_image_source(size=medium_size)
            if name == 'image_%s_source' % size_rname:
                return self.get_image_source(size=size)
        models.Model.__getattribute__(self, name)

    @models.permalink
    def get_absolute_url(self):
        """
        Returns object's absolute URL.
        """
        return ('flickrsets-photo', (), {'flickr_id': self.flickr_id})
    
    def _set_exif(self, d):
        """
        ``_exif`` field setter.
        """
        self._exif = simplejson.dumps(d)
        
    def _get_exif(self):
        """
        ``_exif`` field getter.
        """
        if self._exif:
            return simplejson.loads(self._exif)
        else:
            return {}
    
    exif = property(_get_exif, _set_exif, "Photo EXIF data, as a dictionary.")
    
    def get_image_source(self, size=None):
        """
        Returns object's image source.
        """
        sources = constants.FLICKR_PHOTO_SOURCES
        sizes = [abbrev for abbrev, rname, name in sources]
        if size in sizes and size != constants.FLICKR_PHOTO_SOURCE_MEDIUM:
            secret = self.secret
            if size == constants.FLICKR_PHOTO_SOURCE_ORIGINAL:
                secret = self.original_secret
            return u'http://farm%s.static.flickr.com/%s/%s_%s_%s.jpg' % (
                self.farm,
                self.server,
                self.flickr_id,
                secret,
                size)
        else:
            return u'http://farm%s.static.flickr.com/%s/%s_%s.jpg' % (
                self.farm,
                self.server,
                self.flickr_id,
                self.secret)

    def get_image_url(self, size=None):
        """
        Returns object's image URL.
        """
        urls = constants.FLICKR_PHOTO_URLS
        sizes = [abbrev for abbrev, rname, name in urls]
        if size in sizes:
            return u'%s%s/sizes/%s/' % (
                self.owner.photos_url,
                self.flickr_id,
                size)
        else:
            return u'%s%s/' % (self.owner.photos_url, self.flickr_id)


class Tag(models.Model):
    """
    A Flickr Tag.
    """
    author = models.ForeignKey(
        'flickrsets.Person',
        verbose_name=_('author'),
        related_name='tags')

    flickr_id = models.CharField(
        verbose_name=_('flickr ID'),
        max_length=255)

    name = models.CharField(
        verbose_name=_('name'),
        max_length=255)

    raw = models.CharField(
        verbose_name=_('raw'),
        max_length=255)

    objects = TagManager()

    class Meta:
        """
        Model metadata.
        """
        db_table = '%s_photo_tag' % constants.APP_TABLE_PREFIX
        ordering = ('name',)
        verbose_name = _('tag')
        verbose_name_plural = _('tags')

    def __unicode__(self):
        """
        Object's human readable unicode string.
        """
        return u'%s' % self.name

    @models.permalink
    def get_absolute_url(self):
        """
        Returns object's absolute URL.
        """
        return ('flickrsets-tag', (), {'name': self.name})



"""
Views of Django Flickrsets application.
"""
from django import shortcuts
from django.views.generic import list_detail

from flickrsets import settings as app_settings
from flickrsets.models import Person
from flickrsets.models import Photo
from flickrsets.models import Photoset
from flickrsets.models import Tag


def person_list(request, queryset=None, **kwargs):
    """
    Displays ``Person`` object list.
    
    This is a short wrapper around the generic ``list_detail.object_list`` 
    view. So, all context variables populated by that view will be available 
    here. 
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``Person.objects.all()``
        * ``template_name``: ``flickrsets/person/list.html``
        * ``template_object_name``: ``person``
        * ``paginate_by``: ``settings.FLICKRSETS_PERSON_LIST_VIEW_PAGINATE_BY``
        
    """
    if queryset is None:
        queryset = Person.objects.all()
    
    if 'queryset' in kwargs:
        del kwargs['queryset']
        
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/person/list.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'person'
    
    if 'paginate_by' not in kwargs:
        kwargs['paginate_by'] = getattr(
            app_settings,
            'PERSON_LIST_VIEW_PAGINATE_BY')
            
    return list_detail.object_list(request, queryset, **kwargs)


def person_photo_list(request, flickr_id, queryset=None, **kwargs):
    """
    Displays ``Photo`` object list of a given ``Person`` object.
    
    Takes one required argument:
    
        * ``flickr_id``: the Flickr ID of the given person
        
    This is a short wrapper around the generic ``list_detail.object_list`` 
    view. So, all context variables populated by that view will be available 
    here. One extra variable is added:
    
        * ``person``: the given ``Person`` object
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``person.photos.all()`` (``person`` is the given ``Person`` object)
        * ``template_name``: ``flickrsets/person/photo_list.html``
        * ``template_object_name``: ``photo``
        * ``paginate_by``: ``settings.FLICKRSETS_PERSON_PHOTO_LIST_VIEW_PAGINATE_BY``

    """
    if queryset is None:
        queryset = Person.objects.all()
        
    if 'queryset' in kwargs:
        del kwargs['queryset']
        
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/person/photo_list.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'photo'
    
    if 'paginate_by' not in kwargs:
        kwargs['paginate_by'] = getattr(
            app_settings,
            'PERSON_PHOTO_LIST_VIEW_PAGINATE_BY')
            
    person = shortcuts.get_object_or_404(queryset, flickr_id=flickr_id)

    if 'extra_context' not in kwargs:
        kwargs['extra_context'] = {}
    kwargs['extra_context']['person'] = person
    
    queryset = person.photos.all()
    
    return list_detail.object_list(request, queryset, **kwargs)


def photo_list(request, queryset=None, **kwargs):
    """
    Displays ``Photo`` object list.
    
    This is a short wrapper around the generic ``list_detail.object_list`` 
    view. So, all context variables populated by that view will be available 
    here. 
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``Photo.objects.all()``
        * ``template_name``: ``flickrsets/photo/list.html``
        * ``template_object_name``: ``photo``
        * ``paginate_by``: ``settings.FLICKRSETS_PHOTO_LIST_VIEW_PAGINATE_BY``
        
    """
    if queryset is None:
        queryset = Photo.objects.all()
    
    if 'queryset' in kwargs:
        del kwargs['queryset']
        
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/photo/list.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'photo'
    
    if 'paginate_by' not in kwargs:
        kwargs['paginate_by'] = getattr(
            app_settings,
            'PHOTO_LIST_VIEW_PAGINATE_BY')
            
    return list_detail.object_list(request, queryset, **kwargs)


def photo_detail(request, flickr_id, queryset=None, **kwargs):
    """
    Displays a given ``Photo`` object.
    
    Takes one required argument:
    
        * ``flickr_id``: the given photo Flickr ID
        
    This is a short wrapper around the generic ``list_detail.object_detail`` 
    view. So, all context variables populated by that view will be available 
    here. 
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``Photo.objects.all()``
        * ``slug``: ``flickr_id``
        * ``slug_field``: ``flickr_id``
        * ``template_name``: ``flickrsets/photo/detail.html``
        * ``template_object_name``: ``photo``
        
    """
    if queryset is None:
        queryset = Photo.objects.all()
    
    for key in ('queryset', 'slug'):
        if key in kwargs:
            del kwargs[key]
    
    kwargs['slug'] = flickr_id
    
    if 'slug_field' not in kwargs:
        kwargs['slug_field'] = 'flickr_id'
    
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/photo/detail.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'photo'
    
    return list_detail.object_detail(request, queryset, **kwargs)


def photoset_list(request, queryset=None, **kwargs):
    """
    Displays ``Photoset`` object list.
    
    This is a short wrapper around the generic ``list_detail.object_list`` 
    view. So, all context variables populated by that view will be available 
    here. 
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``Photoset.objects.all()``
        * ``template_name``: ``flickrsets/photoset/list.html``
        * ``template_object_name``: ``photoset``
        * ``paginate_by``: ``settings.FLICKRSETS_PHOTOSET_LIST_VIEW_PAGINATE_BY``
        
    """
    if queryset is None:
        queryset = Photoset.objects.all()
    
    if 'queryset' in kwargs:
        del kwargs['queryset']
        
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/photoset/list.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'photoset'
    
    if 'paginate_by' not in kwargs:
        kwargs['paginate_by'] = getattr(
            app_settings,
            'PHOTOSET_LIST_VIEW_PAGINATE_BY')
            
    return list_detail.object_list(request, queryset, **kwargs)


def photoset_photo_list(request, flickr_id, queryset=None, **kwargs):
    """
    Displays ``Photo`` object list of a given ``Photoset`` object.
    
    Takes one required argument:
    
        * ``flickr_id``: the given photoset Flickr ID
        
    This is a short wrapper around the generic ``list_detail.object_list`` 
    view. So, all context variables populated by that view will be available 
    here. One extra variable is added:
    
        * ``photoset``: the given ``Photoset`` object
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``photoset.photos.all()`` (``photoset`` is the given ``Photoset`` object)
        * ``template_name``: ``flickrsets/photoset/photo_list.html``
        * ``template_object_name``: ``photo``
        * ``paginate_by``: ``settings.FLICKRSETS_PHOTOSET_PHOTO_LIST_VIEW_PAGINATE_BY``

    """
    if queryset is None:
        queryset = Photoset.objects.all()
        
    if 'queryset' in kwargs:
        del kwargs['queryset']
        
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/photoset/photo_list.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'photo'
    
    if 'paginate_by' not in kwargs:
        kwargs['paginate_by'] = getattr(
            app_settings,
            'PHOTOSET_PHOTO_LIST_VIEW_PAGINATE_BY')
            
    photoset = shortcuts.get_object_or_404(queryset, flickr_id=flickr_id)

    if 'extra_context' not in kwargs:
        kwargs['extra_context'] = {}
    kwargs['extra_context']['photoset'] = photoset
    
    queryset = photoset.photos.all()
    
    return list_detail.object_list(request, queryset, **kwargs)


def tag_list(request, queryset=None, **kwargs):
    """
    Displays ``Tag`` object list.
    
    This is a short wrapper around the generic ``list_detail.object_list`` 
    view. So, all context variables populated by that view will be available 
    here. 
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``Tag.objects.all()``
        * ``template_name``: ``flickrsets/tag/list.html``
        * ``template_object_name``: ``tag``

    """
    if queryset is None:
        queryset = Tag.objects.all()
    
    if 'queryset' in kwargs:
        del kwargs['queryset']
        
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/tag/list.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'tag'
            
    return list_detail.object_list(request, queryset, **kwargs)


def tag_photo_list(request, name, queryset=None, **kwargs):
    """
    Displays ``Photo`` object list of a given ``Tag`` object.
    
    Takes one required argument:
    
        * ``name``: the given tag name
        
    This is a short wrapper around the generic ``list_detail.object_list`` 
    view. So, all context variables populated by that view will be available 
    here. One extra variable is added:
    
        * ``tag``: the given ``Tag`` object
    
    Additionally, any keyword arguments which are valid for this generic
    view will be accepted and passed to it.
    
    Default values are:
    
        * ``queryset``: ``tag.photos.all()`` (``tag`` is the given ``Tag`` object)
        * ``template_name``: ``flickrsets/tag/photo_list.html``
        * ``template_object_name``: ``photo``
        * ``paginate_by``: ``settings.FLICKRSETS_TAG_PHOTO_LIST_VIEW_PAGINATE_BY``

    """
    if queryset is None:
        queryset = Tag.objects.all()
        
    if 'queryset' in kwargs:
        del kwargs['queryset']
        
    if 'template_name' not in kwargs:
        kwargs['template_name'] = 'flickrsets/tag/photo_list.html'
    
    if 'template_object_name' not in kwargs:
        kwargs['template_object_name'] = 'photo'
    
    if 'paginate_by' not in kwargs:
        kwargs['paginate_by'] = getattr(
            app_settings,
            'TAG_PHOTO_LIST_VIEW_PAGINATE_BY')
            
    tag = shortcuts.get_object_or_404(queryset, name=name)

    if 'extra_context' not in kwargs:
        kwargs['extra_context'] = {}
    kwargs['extra_context']['tag'] = tag
    
    queryset = tag.photos.all()
    
    return list_detail.object_list(request, queryset, **kwargs)



"""
Tests of Flickr API client of Django Flickrsets application.
"""
from __future__ import with_statement

import mock
import unittest

from flickrsets import utils
from flickrsets.client import FlickrClient
from flickrsets.client import FlickrError


class FlickrClientTest(unittest.TestCase):
    """
    Tests ``FlickrClient`` class.
    """

    def test_client_getattr(self):
        """
        Tests ``__getattr__`` method.
        """
        c1 = FlickrClient('apikey')
        self.assertEquals(c1.api_key, 'apikey')
        self.assertEquals(c1.method, 'flickr')

        c2 = c1.foo.bar.baz
        self.assertEquals(c2.api_key, 'apikey')
        self.assertEquals(c2.method, 'flickr.foo.bar.baz')

    def test_client_call(self):
        """
        Tests ``__call__`` method.
        """
        mock_get_json = mock.Mock(return_value={})
        with mock.patch_object(utils, 'get_json', mock_get_json) as mocked:
            c = FlickrClient('apikey')
            res = c.foo.bar(a=1, b=2)
            self.assert_(mocked.called)

    def test_client_call_fail(self):
        """
        Tests ``__call__`` method which fails.
        """
        failure = {'stat': 'fail', 'code': 1, 'message': 'fail'}
        mock_get_json = mock.Mock(return_value=failure)
        with mock.patch_object(utils, 'get_json', mock_get_json):
            c = FlickrClient('apikey')
            self.assertRaises(FlickrError, c.foo)


# Fake client
# -----------------------------------------------------------------------------
FakeClient = mock.Mock()
FakeClient.return_value = FakeClient

# Person
FakeClient.people.getInfo.return_value = {
    "person": {
        "username": {
            "_content": "gillesfabio"
        },
        "photosurl": {
            "_content": "http://www.flickr.com/photos/gillesfabio/"
        },
        "nsid": "94238521@N00",
        "path_alias": "gillesfabio",
        "photos": {
            "count": {
                "_content": 660
            },
            "firstdatetaken": {
                "_content": "2005-12-10 20:34:19"
            },
            "firstdate": {
                "_content": "1183230521"
            }
        },
        "iconserver": "118",
        "profileurl": {
            "_content": "http://www.flickr.com/people/gillesfabio/"
        },
        "mobileurl": {
            "_content": "http://m.flickr.com/photostream.gne?id=3179235"
        },
        "ispro": 1,
        "location": {
            "_content": "Saint-Laurent-du-Var, France"
        },
        "id": "94238521@N00",
        "realname": {
            "_content": "Gilles Fabio"
        },
        "iconfarm": 1
    },
    "stat": "ok"
}

# Photo
FakeClient.photos.getInfo.return_value = {
    "photo": {
        "dateuploaded": "1262561502",
        "originalformat": "jpg",
        "owner": {
            "username": "gillesfabio",
            "realname": "Gilles Fabio",
            "nsid": "94238521@N00",
            "location": "Saint-Laurent-du-Var, France"
        },
        "id": "4242733600",
        "title": {
            "_content": "Mozart Opera Rock"
        },
        "media": "photo",
        "tags": {
            "tag": [
                {
                    "raw": "bokeh",
                    "machine_tag": 0,
                    "id": "3179235-4242733600-4796",
                    "_content": "bokeh",
                    "author": "94238521@N00"
                },
                {
                    "raw": "mozart",
                    "machine_tag": 0,
                    "id": "3179235-4242733600-44656",
                    "_content": "mozart",
                    "author": "94238521@N00"
                },
                {
                    "raw": "music",
                    "machine_tag": 0,
                    "id": "3179235-4242733600-326",
                    "_content": "music",
                    "author": "94238521@N00"
                },
            ]
        },
        "comments": {
            "_content": "2"
        },
        "secret": "4a3364d8a9",
        "usage": {
            "canblog": 0,
            "canshare": 0,
            "candownload": 1,
            "canprint": 0
        },
        "description": {
            "_content": ""
        },
        "isfavorite": 0,
        "views": "22",
        "farm": 5,
        "rotation": 0,
        "dates": {
            "taken": "2010-01-03 17:15:33",
            "lastupdate": "1275542160",
            "takengranularity": "0",
            "posted": "1262561502"
        },
        "originalsecret": "288ebc9494",
        "license": "0",
        "notes": {
            "note": []
        },
        "server": "4032",
    },
    "stat": "ok"
}

# Exif tags
FakeClient.photos.getExif.return_value = {
    "photo": {
        "farm": 5,
        "secret": "4a3364d8a9",
        "id": "4242733600",
        "exif": [
            {
                "raw": {
                    "_content": "400"
                },
                "tagspace": "ExifIFD",
                "tagspaceid": 0,
                "tag": "ISO",
                "label": "ISO Speed"
            },
            {
                "raw": {
                    "_content": "Gilles Fabio"
                },
                "tagspace": "IPTC",
                "tagspaceid": 0,
                "tag": "CopyrightNotice",
                "label": "Copyright Notice"
            },
        ],
        "server": "4032"
    },
    "stat": "ok"
}

# Tags
FakeClient.tags.getListPhoto.return_value = {
    "photo": {
        "id": "4242733600", 
        "tags": {
            "tag": [
                {
                    "machine_tag": 0, 
                    "_content": "bokeh", 
                    "author": "94238521@N00", 
                    "raw": "bokeh", 
                    "authorname": "gillesfabio", 
                    "id": "3179235-4242733600-4796"
                }, 
                {
                    "machine_tag": 0, 
                    "_content": "mozart", 
                    "author": "94238521@N00", 
                    "raw": "mozart", 
                    "authorname": "gillesfabio", 
                    "id": "3179235-4242733600-44656"
                }, 
                {
                    "machine_tag": 0, 
                    "_content": "music", 
                    "author": "94238521@N00", 
                    "raw": "music", 
                    "authorname": "gillesfabio", 
                    "id": "3179235-4242733600-326"
                }, 
            ]
        }
    }, 
    "stat": "ok"
}

# Photoset
FakeClient.photosets.getInfo.return_value = {
    "stat": "ok",
    "photoset": {
        "description": {
            "_content": ""
        },
        "title": {
            "_content": "Misc"
        },
        "farm": 5,
        "primary": "4242733600",
        "server": "4032",
        "photos": 3,
        "secret": "4a3364d8a9",
        "owner": "94238521@N00",
        "id": "72157623007721343"
    }
}

FakeClient.photosets.getPhotos.return_value = {
    "stat": "ok",
    "photoset": {
        "perpage": 500,
        "photo": [
            {
                "title": "Mozart Opera Rock",
                "farm": 5,
                "server": "4032",
                "secret": "4a3364d8a9",
                "isprimary": "1",
                "id": "4242733600"
            },
            {
                "title": "Self-portrait",
                "farm": 3,
                "server": "2786",
                "secret": "8dc4026bc2",
                "isprimary": "0",
                "id": "4242740086"
            },
            {
                "title": "Candle",
                "farm": 5,
                "server": "4010",
                "secret": "f7dd8201be",
                "isprimary": "0",
                "id": "4242741774"
            }
        ],
        "pages": 1,
        "primary": "4242733600",
        "id": "72157623007721343",
        "ownername": "gillesfabio",
        "owner": "94238521@N00",
        "per_page": 500,
        "total": "3",
        "page": 1
    }
}



"""
Defaults URLs of Django Flickrsets application.
"""
from django.conf.urls.defaults import include
from django.conf.urls.defaults import patterns


urlpatterns = patterns('',
    (r'^people/', include('flickrsets.urls.people')),
    (r'^sets/', include('flickrsets.urls.photosets')),
    (r'^tags/', include('flickrsets.urls.tags')),
    (r'', include('flickrsets.urls.photos')),
)



"""
Settings of Django Flickrsets application.
"""
from django.conf import settings


# Flickr
# -----------------------------------------------------------------------------
FLICKR_API_KEY = getattr(
    settings,
    'FLICKRSETS_FLICKR_API_KEY',
    None)

FLICKR_USER_ID = getattr(
    settings,
    'FLICKRSETS_FLICKR_USER_ID',
    None)

# Tags
# -----------------------------------------------------------------------------
CREATE_TAGS = getattr(
    settings,
    'FLICKRSETS_CREATE_TAGS',
    True)

# EXIF Tags
# -----------------------------------------------------------------------------
SAVE_EXIF_TAGS = getattr(
    settings,
    'FLICKRSETS_SAVE_EXIF_TAGS',
    True)

EXIF_TAG_SPACE_LIST = getattr(
    settings,
    'FLICKRSETS_EXIF_TAG_SPACE_LIST',
    ('ExifIFD', 'IPTC'))

# Views
# -----------------------------------------------------------------------------
PERSON_LIST_VIEW_PAGINATE_BY = getattr(
    settings,
    'FLICKRSETS_PERSON_LIST_VIEW_PAGINATE_BY',
    10)
    
PERSON_PHOTO_LIST_VIEW_PAGINATE_BY = getattr(
    settings,
    'FLICKRSETS_PERSON_PHOTO_LIST_VIEW_PAGINATE_BY',
    10)

PHOTO_LIST_VIEW_PAGINATE_BY = getattr(
    settings,
    'FLICKRSETS_PHOTO_LIST_VIEW_PAGINATE_BY',
    10)

PHOTOSET_LIST_VIEW_PAGINATE_BY = getattr(
    settings,
    'FLICKRSETS_PHOTOSET_LIST_VIEW_PAGINATE_BY',
    10)

PHOTOSET_PHOTO_LIST_VIEW_PAGINATE_BY = getattr(
    settings,
    'FLICKRSETS_PHOTOSET_PHOTO_LIST_VIEW_PAGINATE_BY',
    10)
    
TAG_PHOTO_LIST_VIEW_PAGINATE_BY = getattr(
    settings,
    'FLICKRSETS_TAG_PHOTO_LIST_VIEW_PAGINATE_BY',
    10)

# Synchronizer
# -----------------------------------------------------------------------------
SYNCHRONIZER_PHOTO_TIME_SLEEP = getattr(
    settings,
    'FLICKRSETS_SYNCHRONIZER_PHOTO_TIME_SLEEP',
    1)

SYNCHRONIZER_PHOTOSET_TIME_SLEEP = getattr(
    settings,
    'FLICKRSETS_SYNCHRONIZER_PHOTOSET_TIME_SLEEP',
    1)



"""
Administration classes of Django Flickrsets application.
"""
from django.contrib import admin
from django.core.urlresolvers import reverse
from django.db.models.related import RelatedObject
from django.utils.translation import ugettext_lazy as _

from flickrsets import constants
from flickrsets import models


class BaseModelAdmin(admin.ModelAdmin):
    """
    Model admin base class.
    """

    def get_readonly_fields(self, request, obj=None):
        """
        Returns read-only fields.
        """
        if obj is None:
            return
        new_fields = []
        field_names = obj._meta.get_all_field_names()
        for field_name in field_names:
            field = obj._meta.get_field_by_name(field_name)[0]
            if isinstance(field, RelatedObject):
                continue
            if field.primary_key:
                continue
            new_fields.append(field_name)
        return new_fields


class PersonAdmin(BaseModelAdmin):
    """
    Administration class of ``Person`` model.
    """
    list_display = (
        'obj_buddy_icon',
        'name',
        'flickr_id',
        'username',
        'realname',
        'gender',
        'location',
        'obj_urls')
        
    list_filter = (
        'gender',
        'location')
        
    search_fields = (
        'flickr_id',
        'username',
        'realname',
        'location')

    def obj_buddy_icon(self, obj):
        """
        Returns object's buddy icon.
        """
        return u'<img src="%s" alt="%s" />' % (obj.buddy_icon_url, obj.name)
    obj_buddy_icon.allow_tags = True
    obj_buddy_icon.short_description = _('icon')

    def obj_urls(self, obj):
        """
        Returns object's Flickr's URLs.
        """
        links = []
        links.append(u'<a href="%s">%s</a>' % (obj.photos_url, _('photos')))
        links.append(u'<a href="%s">%s</a>' % (obj.profile_url, _('profile')))
        links.append(u'<a href="%s">%s</a>' % (obj.mobile_url, _('mobile')))
        ul = '<ul>'
        for link in links:
            ul += '<li>%s</li>' % link
        ul += '</ul>'
        return ul
    obj_urls.allow_tags = True
    obj_urls.short_description = _('URLs')
    

class PhotoAdmin(BaseModelAdmin):
    """
    Administration class of ``Photo`` model.
    """
    fieldsets = (
        (_('General'), {
            'fields': (
                'title',
                'flickr_id',
                'tags',
                'description',
                'photoset',
                'owner',
                'media',
                'license',
            ),
        }),
        (_('Dates'), {
            'fields': (
                'date_uploaded',
                'date_posted',
                'date_updated',
                'date_taken',
            ),
        }),
        (_('Counts'), {
            'fields': (
                'views_count',
                'comments_count',
            ),
        }),
        (_('Server'), {
            'fields': (
                'farm',
                'server',
                'secret',
                'original_secret',
                'original_format',
            ),
        })
    )
    list_display = (
        'obj_photo',
        'title',
        'flickr_id',
        'obj_photoset',
        'obj_owner',
        'date_uploaded',
        'views_count',
        'comments_count',
        'obj_image_sources',
        'obj_image_urls')
        
    list_filter = (
        'owner',
        'photoset',
        'date_uploaded',
        'media')
        
    search_fields = (
        'title',
        'description',
        'server',
        'farm')

    def obj_photo(self, obj):
        """
        Returns object's photo preview.
        """
        return u'<img src="%s" alt="%s" />' % (
            obj.image_square_source,
            obj.title)
    obj_photo.allow_tags = True
    obj_photo.short_description = _('photo')

    def obj_photoset(self, obj):
        """
        Returns admin link of related ``Photoset`` object.
        """
        return u'<a href="%s">%s</a>' % (
            reverse(
                'admin:flickrsets_photoset_change',
                args=[obj.photoset.pk]),
            obj.photoset)
    obj_photoset.allow_tags = True
    obj_photoset.short_description = _('Photoset')

    def obj_owner(self, obj):
        """
        Returns admin link of related ``Person`` object.
        """
        return u'<a href="%s">%s</a>' % (
            reverse('admin:flickrsets_person_change', args=[obj.owner.pk]),
            obj.owner)
    obj_owner.allow_tags = True
    obj_owner.short_description = _('owner')

    def obj_image_sources(self, obj):
        """
        Returns object's image sources.
        """
        sources = u'<ul>'
        for abbrev, rname, name in constants.FLICKR_PHOTO_SOURCES:
            source_url = getattr(obj, 'image_%s_source' % rname)
            sources += u'<li><a href="%s">%s</a></li>' % (source_url, name)
        sources += u'</ul>'
        return sources
    obj_image_sources.allow_tags = True
    obj_image_sources.short_description = _('files')

    def obj_image_urls(self, obj):
        """
        Returns object's image URLs.
        """
        urls = u'<ul>'
        for abbrev, rname, name in constants.FLICKR_PHOTO_URLS:
            size_url = getattr(obj, 'image_%s_url' % rname)
            urls += u'<li><a href="%s">%s</a></li>' % (size_url, name)
        urls += u'</ul>'
        return urls
    obj_image_urls.allow_tags = True
    obj_image_urls.short_description = _('URLs')


class PhotosetAdmin(BaseModelAdmin):
    """
    Administration class of ``Photoset`` model.
    """
    list_display = (
        '__unicode__',
        'obj_owner',
        'photos_count')
        
    list_filter = ('owner',)
    
    search_fields = (
        'title',
        'description',
        'server',
        'farm')

    def obj_owner(self, obj):
        """
        Returns admin link of related ``Person`` object (owner)
        """
        return u'<a href="%s">%s</a>' % (
            reverse('admin:flickrsets_person_change', args=[obj.owner.pk]),
            obj.owner.name)
    obj_owner.allow_tags = True
    obj_owner.short_description = _('owner')


class RegisteredSetAdmin(admin.ModelAdmin):
    """
    Administration class of ``RegisteredSet`` model.
    """
    list_display = (
        'title',
        'flickr_id',
        'enabled')


admin.site.register(models.Person, PersonAdmin)
admin.site.register(models.Photo, PhotoAdmin)
admin.site.register(models.Photoset, PhotosetAdmin)
admin.site.register(models.RegisteredSet, RegisteredSetAdmin)



"""
Tests template tags of Django Flickrsets application.
"""
from django import template
from django import test

from flickrsets import constants
from flickrsets.models import Photo


class TemplateTagTest(test.TestCase):
    """
    Helper class.
    """

    def install_tag_library(self, library):
        """
        Installs tag library.
        """
        template.libraries[library] = __import__(library)

    def render_template(self, template_string, **context):
        """
        Renders template.
        """
        t = template.Template(template_string)
        c = template.Context(context)
        return t.render(c)


class PhotoFlickrUrlsNodeTest(TemplateTagTest):
    """
    Tests ``PhotoFlickrUrlsNode`` class.
    """
    fixtures = ['flickrsets/tests']

    def setUp(self):
        """
        Sets up the test.
        """
        self.install_tag_library('flickrsets.templatetags.flickrsets_tags')

    def test_render(self):
        """
        Tests the render.
        """
        photo = Photo.objects.get(flickr_id=constants.TEST_PHOTO_FLICKR_ID)
        items = []
        for abbrev, rname, name in constants.FLICKR_PHOTO_URLS:
            item = {}
            item['title'] = name
            item['url'] = photo.get_image_url(size=abbrev)
            items.append(item)
        should_output = u''
        for item in items:
            should_output += u'%s - %s' % (item['title'], item['url'])
        output = self.render_template((
            "{% load flickrsets_tags %}"
            "{% flickrsets_get_photo_flickr_urls photo %}"
            "{% for url in photo_flickr_urls %}"
            "{{ url.title }} - {{ url.url }}"
            "{% endfor %}"),
            photo=photo)
        self.assertEquals(output, should_output)



"""
URLs of Django Flickrsets application related to ``Person`` model.
"""
from django.conf.urls.defaults import patterns
from django.conf.urls.defaults import url


urlpatterns = patterns('flickrsets.views',
    url(r'^(?P<flickr_id>[-@\w]+)/$',
        view='person_photo_list',
        name='flickrsets-person',
    ),
    url(r'^$',
        view='person_list',
        name='flickrsets-people',
    ),
)



# -*- coding: utf-8 -*-
"""
Setup of Django Flickrsets application.
"""
import os
from setuptools import setup
from setuptools import find_packages

from flickrsets import VERSION


README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(
    name='django-flickrsets',
    version='.'.join([str(n) for n in VERSION]),
    description='Reusable Django application to synchronize Flickr sets.',
    long_description=README,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    keywords='django,flickr,sets,flickrsets,photo',
    author='Gilles Fabio',
    author_email='gilles.fabio@gmail.com',
    url='http://github.com/gillesfabio/django-flickrsets',
    license='BSD',
    packages=find_packages(exclude=['flickrsets_example']),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        'Django >= 1.2',
        'python-dateutil',
        'setuptools',
        'httplib2',
        'South',
        'mock',
    ],
)



from flickrsets.tests.client import FlickrClientTest
from flickrsets.tests.client import FakeClient

from flickrsets.tests.models import PersonTest
from flickrsets.tests.models import PersonManagerTest
from flickrsets.tests.models import PhotoTest
from flickrsets.tests.models import PhotoManagerTest
from flickrsets.tests.models import PhotosetTest
from flickrsets.tests.models import PhotosetManagerTest
from flickrsets.tests.models import TagTest
from flickrsets.tests.models import TagManagerTest

from flickrsets.tests.parsers import PersonParserTest
from flickrsets.tests.parsers import PhotoParserTest
from flickrsets.tests.parsers import PhotosetParserTest
from flickrsets.tests.parsers import PhotoTagsParserTest

from flickrsets.tests.templatetags import PhotoFlickrUrlsNodeTest

from flickrsets.tests.views import ViewsTest
